package org.apidesign.apifest08.currency;

import java.util.Date;

/**
 * Exchange rate provider.
 * 
 * @author ked
 * @see http://wiki.apidesign.org/wiki/APIDesignPatterns:ResponseReply
 */
public interface ExchangeRateProvider<AmountType, IdentifierType> {

    public void getExchangeRate(
            ExchangeRateRequest<AmountType, IdentifierType> request,
            ExchangeRateResponse<AmountType, IdentifierType> response);

    public final class ExchangeRateRequest<AmountType, IdentifierType> {

        private IdentifierType currencyAIdentifier;
        private IdentifierType currencyBIdentifier;
        private Date instant;

        ExchangeRateRequest() {
        }

        public IdentifierType getCurrencyAIdentifier() {
            return currencyAIdentifier;
        }

        void setCurrencyAIdentifier(IdentifierType currencyAIdentifier) {
            this.currencyAIdentifier = currencyAIdentifier;
        }

        public IdentifierType getCurrencyBIdentifier() {
            return currencyBIdentifier;
        }

        void setCurrencyBIdentifier(IdentifierType currencyBIdentifier) {
            this.currencyBIdentifier = currencyBIdentifier;
        }

        public Date getInstant() {
            return instant;
        }

        void setInstant(Date instant) {
            this.instant = instant;
        }
    }

    public final class ExchangeRateResponse<AmountType, IdentifierType> {

        private ExchangeRateValue<AmountType, IdentifierType> exchangeRate;

        ExchangeRateResponse() {
        }

        ExchangeRateValue<AmountType, IdentifierType> getExchangeRate() {
            return exchangeRate;
        }

        public void setExchangeRate(ExchangeRateValue<AmountType, IdentifierType> exchangeRate) {
            this.exchangeRate = exchangeRate;
        }
    }
}
