package org.apidesign.apifest08.currency;

/**
 * Static exchange rate provider.
 * 
 * @author ked
 */
final class StaticExchangeRateProvider<AmountType, IdentifierType> implements ExchangeRateProvider<AmountType, IdentifierType> {

    final ExchangeRateValue<AmountType, IdentifierType> exchangeRate;

    private StaticExchangeRateProvider(ExchangeRateValue<AmountType, IdentifierType> exchangeRate) {
        this.exchangeRate = exchangeRate;
    }

    public void getExchangeRate(ExchangeRateRequest<AmountType, IdentifierType> request, ExchangeRateResponse<AmountType, IdentifierType> response) {
        response.setExchangeRate(exchangeRate);
    }

    static <AmountType, IdentifierType> StaticExchangeRateProvider<AmountType, IdentifierType> getStaticExchangeRateProvider(ExchangeRateValue<AmountType, IdentifierType> exchangeRate) {
        return new StaticExchangeRateProvider<AmountType, IdentifierType>(exchangeRate);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }

        if (getClass() != obj.getClass()) {
            return false;
        }

        final StaticExchangeRateProvider other = (StaticExchangeRateProvider) obj;
        if (this.exchangeRate != other.exchangeRate && (this.exchangeRate == null || !this.exchangeRate.equals(other.exchangeRate))) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 67 * hash + (this.exchangeRate != null ? this.exchangeRate.hashCode() : 0);
        return hash;
    }
}
