/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.apidesign.apifest08.test;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import junit.framework.TestCase;
import org.apidesign.apifest08.currency.ConversionResult;
import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.ConvertorCurrency;
import org.apidesign.apifest08.currency.ExchangeRate;
import org.apidesign.apifest08.currency.ExchangeRateProvider;
import org.apidesign.apifest08.currency.IDateExchangeRateEngine;
import org.apidesign.apifest08.currency.IDateProviderEngine;

/**
 *
 * @author arnostvalicek
 */
public class ConvertorWithDateExchangeRateTest extends TestCase {

    private static ConvertorCurrency CZK = ConvertorCurrency.getInstance("CZK");
    private static ConvertorCurrency SKK = ConvertorCurrency.getInstance("SKK");
    private static ConvertorCurrency USD = ConvertorCurrency.getInstance("USD");
    private SimpleDateFormat df;

    @Override
    protected void setUp() throws Exception {
        super.setUp();
        df = new SimpleDateFormat("yyyy-MM-dd HH:mm zzzz");
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
        df = null;
    }

    private IDateExchangeRateEngine createDateEngine() {
        IDateExchangeRateEngine engine = new IDateExchangeRateEngine() {

            public ExchangeRate getExchangeRate(ConvertorCurrency fromCurrency, ConvertorCurrency toCurrency, Date date) {
                int day = date.getDay();
                if (day >= 1 && day <= 5) {
                    //weekday
                    return new ExchangeRate(new BigDecimal(1), new BigDecimal(2));
                } else {
                    //weekend
                    return new ExchangeRate(new BigDecimal(1), new BigDecimal(3));
                }
            }
        };
        return engine;
    }

    private Convertor createConvertor() {
        ExchangeRateProvider exchangeRateProvider = ExchangeRateProvider.createDateExchangeRateProvider(createDateEngine());
        Convertor c = Convertor.createConvertor(exchangeRateProvider);
        return c;
    }

    public void testConvertuUsingDifferentRatesForDays() throws ParseException {
        Convertor c = createConvertor();
        {
            Date d1 = df.parse("2008-10-15 9:00 GMT");  //Wednesday
            ConversionResult resultWeekday = c.convert(CZK, SKK, new BigDecimal("2.3"), d1);
            assertEquals("Weekday conversion", new BigDecimal("4.60"), resultWeekday.getConverted());
        }
        {
            Date d1 = df.parse("2008-10-18 9:00 GMT");  //Saturday
            ConversionResult resultWeekday = c.convert(CZK, SKK, new BigDecimal("2.3"), d1);
            assertEquals("Weekday conversion", new BigDecimal("6.90"), resultWeekday.getConverted());
        }
    }

    public void testConvertUsingDifferenDefaultDay() throws ParseException {
        class MyDateProvider implements IDateProviderEngine {

            private Date date;

            void setDate(Date date) {
                this.date = date;
            }

            public Date getCurrentDate() {
                return date;
            }
        }

        Convertor c = createConvertor();
        MyDateProvider dp = new MyDateProvider();
        c.setDateProvider(dp);

        {
            dp.setDate(df.parse("2008-10-15 9:00 GMT")); //Wednesday
            ConversionResult resultWeekday = c.convert(CZK, SKK, new BigDecimal("2.3"));
            assertEquals("Weekday conversion", new BigDecimal("4.60"), resultWeekday.getConverted());
        }
        {
            dp.setDate(df.parse("2008-10-18 9:00 GMT"));  //Saturday
            ConversionResult resultWeekday = c.convert(CZK, SKK, new BigDecimal("2.3"));
            assertEquals("Weekday conversion", new BigDecimal("6.90"), resultWeekday.getConverted());
        }
    }
}
